from typing import TYPE_CHECKING, List

import dagster._check as check
import graphene
from dagster._core.definitions.selector import ResourceSelector
from dagster._core.host_representation.external import ExternalRepository, ExternalResource
from dagster._core.host_representation.external_data import (
    ExternalResourceConfigEnvVar,
    ExternalResourceValue,
    NestedResourceType,
    ResourceJobUsageEntry,
)

from dagster_graphql.schema.asset_key import GrapheneAssetKey
from dagster_graphql.schema.errors import (
    GraphenePythonError,
    GrapheneRepositoryNotFoundError,
    GrapheneResourceNotFoundError,
)
from dagster_graphql.schema.pipelines.pipeline import GrapheneJob
from dagster_graphql.schema.solids import GrapheneSolidHandle, build_solid_handles
from dagster_graphql.schema.util import non_null_list

from .config_types import GrapheneConfigTypeField

if TYPE_CHECKING:
    from dagster._core.workspace.workspace import IWorkspace


class GrapheneConfiguredValueType(graphene.Enum):
    VALUE = "VALUE"
    ENV_VAR = "ENV_VAR"

    class Meta:
        name = "ConfiguredValueType"


class GrapheneConfiguredValue(graphene.ObjectType):
    key = graphene.NonNull(graphene.String)
    value = graphene.NonNull(graphene.String)
    type = graphene.NonNull(GrapheneConfiguredValueType)

    class Meta:
        name = "ConfiguredValue"

    def __init__(self, key: str, external_resource_value: ExternalResourceValue):
        super().__init__()

        self.key = key
        if isinstance(external_resource_value, ExternalResourceConfigEnvVar):
            self.type = GrapheneConfiguredValueType.ENV_VAR
            self.value = external_resource_value.name
        else:
            self.type = GrapheneConfiguredValueType.VALUE
            self.value = external_resource_value


GrapheneNestedResourceType = graphene.Enum.from_enum(NestedResourceType)


class GrapheneNestedResourceEntry(graphene.ObjectType):
    name = graphene.NonNull(graphene.String)
    type = graphene.NonNull(GrapheneNestedResourceType)
    resource = graphene.Field(lambda: GrapheneResourceDetails)

    class Meta:
        name = "NestedResourceEntry"


class GrapheneJobAndSpecificOps(graphene.ObjectType):
    job = graphene.NonNull(GrapheneJob)
    opsUsing = graphene.Field(non_null_list(GrapheneSolidHandle))

    class Meta:
        name = "JobWithOps"


class GrapheneResourceDetails(graphene.ObjectType):
    name = graphene.NonNull(graphene.String)
    description = graphene.String()
    configFields = graphene.Field(
        non_null_list(GrapheneConfigTypeField),
        description="Snapshots of all the fields for the given resource",
    )
    configuredValues = graphene.Field(
        non_null_list(GrapheneConfiguredValue),
        description=(
            "List of K/V pairs of user-configured values for each of the top-level fields on the"
            " resource"
        ),
    )
    isTopLevel = graphene.NonNull(graphene.Boolean)
    nestedResources = graphene.Field(
        non_null_list(GrapheneNestedResourceEntry),
        description="List of nested resources for the given resource",
    )
    parentResources = graphene.Field(
        non_null_list(GrapheneNestedResourceEntry),
        description="List of parent resources for the given resource",
    )
    resourceType = graphene.NonNull(graphene.String)
    assetKeysUsing = graphene.Field(non_null_list(GrapheneAssetKey))
    jobsOpsUsing = graphene.Field(non_null_list(GrapheneJobAndSpecificOps))

    class Meta:
        name = "ResourceDetails"

    def __init__(
        self, location_name: str, repository_name: str, external_resource: ExternalResource
    ):
        super().__init__()

        self._location_name = check.str_param(location_name, "location_name")
        self._repository_name = check.str_param(repository_name, "repository_name")

        self._external_resource = check.inst_param(
            external_resource, "external_resource", ExternalResource
        )
        self.name = external_resource.name
        self.description = external_resource.description
        self._config_field_snaps = external_resource.config_field_snaps
        self._configured_values = external_resource.configured_values

        self._config_schema_snap = external_resource.config_schema_snap
        self.isTopLevel = external_resource.is_top_level
        self._nested_resources = external_resource.nested_resources
        self._parent_resources = external_resource.parent_resources
        self.resourceType = external_resource.resource_type
        self._asset_keys_using = external_resource.asset_keys_using
        self._job_ops_using = external_resource.job_ops_using

    def resolve_configFields(self, _graphene_info):
        return [
            GrapheneConfigTypeField(
                config_schema_snapshot=self._config_schema_snap,
                field_snap=field_snap,
            )
            for field_snap in self._config_field_snaps
        ]

    def resolve_configuredValues(self, _graphene_info):
        return [
            GrapheneConfiguredValue(key=key, external_resource_value=value)
            for key, value in self._configured_values.items()
        ]

    def resolve_nestedResources(self, graphene_info) -> List[GrapheneNestedResourceEntry]:
        from dagster_graphql.implementation.fetch_resources import get_resource_or_error

        return [
            GrapheneNestedResourceEntry(
                name=k,
                type=v.type,
                resource=(
                    get_resource_or_error(
                        graphene_info,
                        ResourceSelector(
                            location_name=self._location_name,
                            repository_name=self._repository_name,
                            resource_name=v.name,
                        ),
                    )
                    if v.type == NestedResourceType.TOP_LEVEL
                    else None
                ),
            )
            for k, v in self._nested_resources.items()
        ]

    def resolve_parentResources(self, graphene_info) -> List[GrapheneNestedResourceEntry]:
        from dagster_graphql.implementation.fetch_resources import get_resource_or_error

        return [
            GrapheneNestedResourceEntry(
                name=attribute,
                type=NestedResourceType.TOP_LEVEL,
                resource=get_resource_or_error(
                    graphene_info,
                    ResourceSelector(
                        location_name=self._location_name,
                        repository_name=self._repository_name,
                        resource_name=name,
                    ),
                ),
            )
            for name, attribute in self._parent_resources.items()
        ]

    def resolve_assetKeysUsing(self, _graphene_info) -> List[GrapheneAssetKey]:
        return [GrapheneAssetKey(path=asset_key.path) for asset_key in self._asset_keys_using]

    def _construct_job_and_specific_ops(
        self, repo: ExternalRepository, entry: ResourceJobUsageEntry
    ) -> GrapheneJobAndSpecificOps:
        job = repo.get_full_external_job(entry.job_name)
        handle_strs = {handle.to_string() for handle in entry.node_handles}
        node_handles = [
            handle
            for handle_str, handle in build_solid_handles(job).items()
            if handle_str in handle_strs
        ]
        return GrapheneJobAndSpecificOps(job=GrapheneJob(job), opsUsing=node_handles)

    def resolve_jobsOpsUsing(self, graphene_info) -> List[GrapheneJobAndSpecificOps]:
        context: IWorkspace = graphene_info.context
        repo = context.get_code_location(self._location_name).get_repository(self._repository_name)

        return [self._construct_job_and_specific_ops(repo, entry) for entry in self._job_ops_using]


class GrapheneResourceDetailsOrError(graphene.Union):
    class Meta:
        types = (GrapheneResourceDetails, GrapheneResourceNotFoundError, GraphenePythonError)
        name = "ResourceDetailsOrError"


class GrapheneResourceDetailsList(graphene.ObjectType):
    results = non_null_list(GrapheneResourceDetails)

    class Meta:
        name = "ResourceDetailsList"


class GrapheneResourceDetailsListOrError(graphene.Union):
    class Meta:
        types = (GrapheneResourceDetailsList, GrapheneRepositoryNotFoundError, GraphenePythonError)
        name = "ResourcesOrError"



from typing import TYPE_CHECKING, Optional

from dagster import AssetKey
from dagster._daemon.asset_daemon import get_current_evaluation_id

from dagster_graphql.implementation.fetch_assets import get_asset_nodes_by_asset_key
from dagster_graphql.schema.auto_materialize_asset_evaluations import (
    GrapheneAutoMaterializeAssetEvaluationNeedsMigrationError,
    GrapheneAutoMaterializeAssetEvaluationRecord,
    GrapheneAutoMaterializeAssetEvaluationRecords,
)
from dagster_graphql.schema.inputs import GrapheneAssetKeyInput

if TYPE_CHECKING:
    from ..schema.util import ResolveInfo


def fetch_auto_materialize_asset_evaluations(
    graphene_info: "ResolveInfo",
    graphene_asset_key: GrapheneAssetKeyInput,
    limit: int,
    cursor: Optional[str],
):
    """Fetch asset policy evaluations from storage."""
    if graphene_info.context.instance.schedule_storage is None:
        return GrapheneAutoMaterializeAssetEvaluationNeedsMigrationError(
            message="Instance does not have schedule storage configured, cannot fetch evaluations."
        )
    if (
        not graphene_info.context.instance.schedule_storage.supports_auto_materialize_asset_evaluations
    ):
        return GrapheneAutoMaterializeAssetEvaluationNeedsMigrationError(
            message=(
                "Auto materialize evaluations are not getting logged. Run `dagster instance"
                " migrate` to enable."
            )
        )

    asset_key = AssetKey.from_graphql_input(graphene_asset_key)
    asset_node = get_asset_nodes_by_asset_key(graphene_info).get(asset_key)
    partitions_def = (
        asset_node.external_asset_node.partitions_def_data.get_partitions_definition()
        if asset_node and asset_node.external_asset_node.partitions_def_data
        else None
    )

    current_evaluation_id = get_current_evaluation_id(graphene_info.context.instance)

    return GrapheneAutoMaterializeAssetEvaluationRecords(
        records=[
            GrapheneAutoMaterializeAssetEvaluationRecord(record, partitions_def=partitions_def)
            for record in graphene_info.context.instance.schedule_storage.get_auto_materialize_asset_evaluations(
                asset_key=asset_key,
                limit=limit,
                cursor=int(cursor) if cursor else None,
            )
        ],
        currentEvaluationId=current_evaluation_id,
    )



import dagster._check as check
import graphene
from dagster._core.definitions.auto_materialize_policy import (
    AutoMaterializePolicy,
    AutoMaterializePolicyType,
)
from dagster._core.definitions.auto_materialize_rule import (
    AutoMaterializeDecisionType,
    AutoMaterializeRuleSnapshot,
    DiscardOnMaxMaterializationsExceededRule,
)

from .util import non_null_list

GrapheneAutoMaterializeDecisionType = graphene.Enum.from_enum(AutoMaterializeDecisionType)


class GrapheneAutoMaterializeRule(graphene.ObjectType):
    description = graphene.NonNull(graphene.String)
    decisionType = graphene.NonNull(GrapheneAutoMaterializeDecisionType)

    class Meta:
        name = "AutoMaterializeRule"

    def __init__(self, auto_materialize_rule_snapshot: AutoMaterializeRuleSnapshot):
        super().__init__(
            decisionType=auto_materialize_rule_snapshot.decision_type,
            description=auto_materialize_rule_snapshot.description,
        )


class GrapheneAutoMaterializePolicy(graphene.ObjectType):
    policyType = graphene.NonNull(graphene.Enum.from_enum(AutoMaterializePolicyType))
    maxMaterializationsPerMinute = graphene.Int()
    rules = non_null_list(GrapheneAutoMaterializeRule)

    class Meta:
        name = "AutoMaterializePolicy"

    def __init__(self, auto_materialize_policy: AutoMaterializePolicy):
        auto_materialize_policy = check.inst_param(
            auto_materialize_policy, "auto_materialize_policy", AutoMaterializePolicy
        )
        # for now, we don't represent the max materializations per minute rule as a proper
        # rule in the serialized AutoMaterializePolicy object, but do so in the GraphQL layer
        rules = [
            GrapheneAutoMaterializeRule(rule.to_snapshot())
            for rule in auto_materialize_policy.rules
        ]
        if auto_materialize_policy.max_materializations_per_minute:
            rules.append(
                GrapheneAutoMaterializeRule(
                    DiscardOnMaxMaterializationsExceededRule(
                        limit=auto_materialize_policy.max_materializations_per_minute
                    ).to_snapshot()
                )
            )
        super().__init__(
            rules=rules,
            policyType=auto_materialize_policy.policy_type,
            maxMaterializationsPerMinute=auto_materialize_policy.max_materializations_per_minute,
        )



from collections import defaultdict
from typing import Optional, Sequence, Tuple

import dagster._check as check
import graphene
from dagster import PartitionsDefinition
from dagster._core.definitions.auto_materialize_rule import (
    AutoMaterializeDecisionType,
    AutoMaterializeRuleEvaluation,
    AutoMaterializeRuleEvaluationData,
    ParentUpdatedRuleEvaluationData,
    TextRuleEvaluationData,
    WaitingOnAssetsRuleEvaluationData,
)
from dagster._core.definitions.partition import SerializedPartitionsSubset
from dagster._core.scheduler.instigation import AutoMaterializeAssetEvaluationRecord

from dagster_graphql.schema.errors import GrapheneError

from .asset_key import GrapheneAssetKey
from .auto_materialize_policy import GrapheneAutoMaterializeRule
from .util import non_null_list

GrapheneAutoMaterializeDecisionType = graphene.Enum.from_enum(AutoMaterializeDecisionType)


class GraphenePartitionKeys(graphene.ObjectType):
    partitionKeys = non_null_list(graphene.String)

    class Meta:
        name = "PartitionKeys"


class GraphenePartitionSubsetDeserializationError(graphene.ObjectType):
    message = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneError,)
        name = "PartitionSubsetDeserializationError"


class GraphenePartitionKeysOrError(graphene.Union):
    class Meta:
        types = (GraphenePartitionKeys, GraphenePartitionSubsetDeserializationError)
        name = "PartitionKeysOrError"


class GrapheneTextRuleEvaluationData(graphene.ObjectType):
    text = graphene.String()

    class Meta:
        name = "TextRuleEvaluationData"


class GrapheneParentMaterializedRuleEvaluationData(graphene.ObjectType):
    updatedAssetKeys = graphene.List(graphene.NonNull(GrapheneAssetKey))
    willUpdateAssetKeys = graphene.List(graphene.NonNull(GrapheneAssetKey))

    class Meta:
        name = "ParentMaterializedRuleEvaluationData"


class GrapheneWaitingOnKeysRuleEvaluationData(graphene.ObjectType):
    waitingOnAssetKeys = graphene.List(graphene.NonNull(GrapheneAssetKey))

    class Meta:
        name = "WaitingOnKeysRuleEvaluationData"


class GrapheneAutoMaterializeRuleEvaluationData(graphene.Union):
    class Meta:
        types = (
            GrapheneTextRuleEvaluationData,
            GrapheneParentMaterializedRuleEvaluationData,
            GrapheneWaitingOnKeysRuleEvaluationData,
        )
        name = "AutoMaterializeRuleEvaluationData"


class GrapheneAutoMaterializeRuleEvaluation(graphene.ObjectType):
    partitionKeysOrError = graphene.Field(GraphenePartitionKeysOrError)
    evaluationData = graphene.Field(GrapheneAutoMaterializeRuleEvaluationData)

    class Meta:
        name = "AutoMaterializeRuleEvaluation"


class GrapheneAutoMaterializeRuleWithRuleEvaluations(graphene.ObjectType):
    rule = graphene.NonNull(GrapheneAutoMaterializeRule)
    ruleEvaluations = non_null_list(GrapheneAutoMaterializeRuleEvaluation)

    class Meta:
        name = "AutoMaterializeRuleWithRuleEvaluations"

    def __init__(
        self,
        rule: GrapheneAutoMaterializeRule,
        ruleEvaluations,
    ):
        super().__init__(
            rule=rule,
            ruleEvaluations=ruleEvaluations,
        )


def create_graphene_auto_materialize_rule_evaluation(
    evaluation_data_tuple: Tuple[
        AutoMaterializeRuleEvaluationData,
        Optional[SerializedPartitionsSubset],
    ],
    partitions_def: Optional[PartitionsDefinition],
):
    rule_evaluation_data, serialized_partition_subset = evaluation_data_tuple

    if not serialized_partition_subset:
        partition_keys_or_error = None
    elif not partitions_def:
        partition_keys_or_error = GraphenePartitionSubsetDeserializationError(
            message="PartitionsDefinition not found, cannot display partition keys"
        )
    elif not serialized_partition_subset.can_deserialize(partitions_def):
        partition_keys_or_error = GraphenePartitionSubsetDeserializationError(
            message=(
                "Partition subset cannot be deserialized. The PartitionsDefinition may have"
                " changed."
            )
        )
    else:
        subset = serialized_partition_subset.deserialize(partitions_def)
        partition_keys_or_error = GraphenePartitionKeys(partitionKeys=subset.get_partition_keys())

    if isinstance(rule_evaluation_data, TextRuleEvaluationData):
        rule_evaluation_data = GrapheneTextRuleEvaluationData(text=rule_evaluation_data.text)
    elif isinstance(rule_evaluation_data, ParentUpdatedRuleEvaluationData):
        rule_evaluation_data = GrapheneParentMaterializedRuleEvaluationData(
            updatedAssetKeys=rule_evaluation_data.updated_asset_keys,
            willUpdateAssetKeys=rule_evaluation_data.will_update_asset_keys,
        )
    elif isinstance(rule_evaluation_data, WaitingOnAssetsRuleEvaluationData):
        rule_evaluation_data = GrapheneWaitingOnKeysRuleEvaluationData(
            waitingOnAssetKeys=rule_evaluation_data.waiting_on_asset_keys
        )
    elif rule_evaluation_data is not None:
        check.failed(f"Unexpected rule evaluation data type {type(rule_evaluation_data)}")

    return GrapheneAutoMaterializeRuleEvaluation(
        partitionKeysOrError=partition_keys_or_error, evaluationData=rule_evaluation_data
    )


def create_graphene_auto_materialize_rules_with_rule_evaluations(
    partition_subsets_by_condition: Sequence[
        Tuple[AutoMaterializeRuleEvaluation, Optional[SerializedPartitionsSubset]]
    ],
    partitions_def: Optional[PartitionsDefinition],
) -> Sequence[GrapheneAutoMaterializeRuleWithRuleEvaluations]:
    rule_mapping = defaultdict(list)
    for rule_evaluation, serialized_partition_subset in partition_subsets_by_condition:
        rule_mapping[rule_evaluation.rule_snapshot].append(
            (rule_evaluation.evaluation_data, serialized_partition_subset)
        )

    return [
        GrapheneAutoMaterializeRuleWithRuleEvaluations(
            rule=GrapheneAutoMaterializeRule(rule_snapshot),
            ruleEvaluations=[
                create_graphene_auto_materialize_rule_evaluation(tup, partitions_def)
                for tup in tups
            ],
        )
        for rule_snapshot, tups in rule_mapping.items()
    ]


class GrapheneAutoMaterializeAssetEvaluationRecord(graphene.ObjectType):
    id = graphene.NonNull(graphene.ID)
    evaluationId = graphene.NonNull(graphene.Int)
    numRequested = graphene.NonNull(graphene.Int)
    numSkipped = graphene.NonNull(graphene.Int)
    numDiscarded = graphene.NonNull(graphene.Int)
    rulesWithRuleEvaluations = non_null_list(GrapheneAutoMaterializeRuleWithRuleEvaluations)
    timestamp = graphene.NonNull(graphene.Float)
    runIds = non_null_list(graphene.String)

    class Meta:
        name = "AutoMaterializeAssetEvaluationRecord"

    def __init__(
        self,
        record: AutoMaterializeAssetEvaluationRecord,
        partitions_def: Optional[PartitionsDefinition],
    ):
        super().__init__(
            id=record.id,
            evaluationId=record.evaluation_id,
            numRequested=record.evaluation.num_requested,
            numSkipped=record.evaluation.num_skipped,
            numDiscarded=record.evaluation.num_discarded,
            rulesWithRuleEvaluations=create_graphene_auto_materialize_rules_with_rule_evaluations(
                record.evaluation.partition_subsets_by_condition, partitions_def
            ),
            timestamp=record.timestamp,
            runIds=record.evaluation.run_ids,
        )


class GrapheneAutoMaterializeAssetEvaluationRecords(graphene.ObjectType):
    records = non_null_list(GrapheneAutoMaterializeAssetEvaluationRecord)
    currentEvaluationId = graphene.Int()

    class Meta:
        name = "AutoMaterializeAssetEvaluationRecords"


class GrapheneAutoMaterializeAssetEvaluationNeedsMigrationError(graphene.ObjectType):
    message = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneError,)
        name = "AutoMaterializeAssetEvaluationNeedsMigrationError"


class GrapheneAutoMaterializeAssetEvaluationRecordsOrError(graphene.Union):
    class Meta:
        types = (
            GrapheneAutoMaterializeAssetEvaluationRecords,
            GrapheneAutoMaterializeAssetEvaluationNeedsMigrationError,
        )
        name = "AutoMaterializeAssetEvaluationRecordsOrError"



from typing import List, Sequence

import graphene
from dagster._core.host_representation.external_data import (
    EnvVarConsumer,
)

from dagster_graphql.schema.errors import (
    GraphenePythonError,
)
from dagster_graphql.schema.util import non_null_list


class GrapheneEnvVarConsumerType(graphene.Enum):
    RESOURCE = "RESOURCE"

    class Meta:
        name = "EnvVarConsumerType"


class GrapheneEnvVarConsumer(graphene.ObjectType):
    type = graphene.NonNull(GrapheneEnvVarConsumerType)
    name = graphene.NonNull(graphene.String)

    class Meta:
        name = "EnvVarConsumer"

    def __init__(self, env_var_consumer: EnvVarConsumer):
        super().__init__()

        self.type = GrapheneEnvVarConsumerType.RESOURCE
        self.name = env_var_consumer.name


class GrapheneEnvVarWithConsumers(graphene.ObjectType):
    envVarName = graphene.NonNull(graphene.String)
    envVarConsumers = graphene.Field(
        non_null_list(GrapheneEnvVarConsumer),
    )

    class Meta:
        name = "EnvVarWithConsumers"

    def __init__(self, name: str, consumers: Sequence[EnvVarConsumer]):
        super().__init__()

        self.envVarName = name
        self.consumers = consumers

    def resolve_envVarConsumers(self, _graphene_info) -> List[GrapheneEnvVarConsumer]:
        return [GrapheneEnvVarConsumer(consumer) for consumer in self.consumers]


class GrapheneEnvVarWithConsumersList(graphene.ObjectType):
    results = non_null_list(GrapheneEnvVarWithConsumers)

    class Meta:
        name = "EnvVarWithConsumersList"


class GrapheneEnvVarWithConsumersListOrError(graphene.Union):
    class Meta:
        types = (GrapheneEnvVarWithConsumersList, GraphenePythonError)
        name = "EnvVarWithConsumersOrError"



from typing import TYPE_CHECKING, cast

import dagster._check as check
from dagster._core.definitions.selector import JobSubsetSelector
from dagster._core.execution.plan.resume_retry import ReexecutionStrategy
from dagster._core.storage.dagster_run import DagsterRun, RunsFilter
from dagster._core.workspace.permissions import Permissions

from ..external import get_external_job_or_raise
from ..utils import (
    ExecutionMetadata,
    ExecutionParams,
    assert_permission_for_location,
)
from .run_lifecycle import create_valid_pipeline_run

if TYPE_CHECKING:
    from dagster._core.instance import DagsterInstance

    from dagster_graphql.schema.runs import GrapheneLaunchRunSuccess
    from dagster_graphql.schema.util import ResolveInfo


def launch_pipeline_reexecution(
    graphene_info: "ResolveInfo", execution_params: ExecutionParams
) -> "GrapheneLaunchRunSuccess":
    return _launch_pipeline_execution(graphene_info, execution_params, is_reexecuted=True)


def launch_pipeline_execution(
    graphene_info: "ResolveInfo", execution_params: ExecutionParams
) -> "GrapheneLaunchRunSuccess":
    return _launch_pipeline_execution(graphene_info, execution_params)


def do_launch(
    graphene_info: "ResolveInfo", execution_params: ExecutionParams, is_reexecuted: bool = False
) -> DagsterRun:
    check.inst_param(execution_params, "execution_params", ExecutionParams)
    check.bool_param(is_reexecuted, "is_reexecuted")

    if is_reexecuted:
        # required fields for re-execution
        execution_metadata = check.inst_param(
            execution_params.execution_metadata, "execution_metadata", ExecutionMetadata
        )
        check.str_param(execution_metadata.root_run_id, "root_run_id")
        check.str_param(execution_metadata.parent_run_id, "parent_run_id")
    external_job = get_external_job_or_raise(graphene_info, execution_params.selector)
    dagster_run = create_valid_pipeline_run(graphene_info, external_job, execution_params)

    return graphene_info.context.instance.submit_run(
        dagster_run.run_id,
        workspace=graphene_info.context,
    )


def _launch_pipeline_execution(
    graphene_info: "ResolveInfo", execution_params: ExecutionParams, is_reexecuted: bool = False
) -> "GrapheneLaunchRunSuccess":
    from ...schema.pipelines.pipeline import GrapheneRun
    from ...schema.runs import GrapheneLaunchRunSuccess

    check.inst_param(execution_params, "execution_params", ExecutionParams)
    check.bool_param(is_reexecuted, "is_reexecuted")

    run = do_launch(graphene_info, execution_params, is_reexecuted)
    records = graphene_info.context.instance.get_run_records(RunsFilter(run_ids=[run.run_id]))

    return GrapheneLaunchRunSuccess(run=GrapheneRun(records[0]))


def launch_reexecution_from_parent_run(
    graphene_info: "ResolveInfo", parent_run_id: str, strategy: str
) -> "GrapheneLaunchRunSuccess":
    """Launch a re-execution by referencing the parent run id."""
    from ...schema.pipelines.pipeline import GrapheneRun
    from ...schema.runs import GrapheneLaunchRunSuccess

    check.str_param(parent_run_id, "parent_run_id")

    instance: DagsterInstance = graphene_info.context.instance
    parent_run = check.not_none(
        instance.get_run_by_id(parent_run_id), f"Could not find parent run with id: {parent_run_id}"
    )
    origin = check.not_none(parent_run.external_job_origin)
    selector = JobSubsetSelector(
        location_name=origin.external_repository_origin.code_location_origin.location_name,
        repository_name=origin.external_repository_origin.repository_name,
        job_name=parent_run.job_name,
        op_selection=None,
    )

    assert_permission_for_location(
        graphene_info,
        Permissions.LAUNCH_PIPELINE_REEXECUTION,
        selector.location_name,
    )

    repo_location = graphene_info.context.get_code_location(selector.location_name)
    external_pipeline = get_external_job_or_raise(graphene_info, selector)

    run = instance.create_reexecuted_run(
        parent_run=cast(DagsterRun, parent_run),
        code_location=repo_location,
        external_job=external_pipeline,
        strategy=ReexecutionStrategy(strategy),
        use_parent_run_tags=True,  # inherit whatever tags were set on the parent run at launch time
    )
    graphene_info.context.instance.submit_run(
        run.run_id,
        workspace=graphene_info.context,
    )

    # return run with updateTime
    records = graphene_info.context.instance.get_run_records(RunsFilter(run_ids=[run.run_id]))
    return GrapheneLaunchRunSuccess(run=GrapheneRun(records[0]))



import graphene

from ..errors import GrapheneInvalidSubsetError, GraphenePipelineNotFoundError, GraphenePythonError
from ..execution import GrapheneExecutionPlan
from ..pipelines.config import GrapheneRunConfigValidationInvalid


class GrapheneExecutionPlanOrError(graphene.Union):
    class Meta:
        types = (
            GrapheneExecutionPlan,
            GrapheneRunConfigValidationInvalid,
            GraphenePipelineNotFoundError,
            GrapheneInvalidSubsetError,
            GraphenePythonError,
        )
        name = "ExecutionPlanOrError"
